from sedaro_base_client.paths.simulations_branches_branch_id_control_job_id.get import ApiForget
from sedaro_base_client.paths.simulations_branches_branch_id_control_job_id.delete import ApiFordelete


class SimulationsBranchesBranchIdControlJobId(
    ApiForget,
    ApiFordelete,
):
    pass
