from sedaro_base_client.paths.simulations_branches_branch_id_control_study_.get import ApiForget
from sedaro_base_client.paths.simulations_branches_branch_id_control_study_.post import ApiForpost


class SimulationsBranchesBranchIdControlStudy(
    ApiForget,
    ApiForpost,
):
    pass
